class PostsController < ApplicationController
  before_action :set_post, only: %i[ show update destroy ]

  # GET /posts
  def index
    if params[:tags].present?
      tag_names = params[:tags].split(',')
      match_type = params[:match]
      if match_type == 'any'
        @posts = Post.joins(:tags).where(tags: { name: tag_names }).distinct
      else
        @posts = Post.joins(:tags)
        tag_names.each do |tag|
          @posts = @posts.where(id: Post.joins(:tags).where(tags: { name: tag }))
        end
        @posts = @posts.distinct
      end
    else
      @posts = Post.all
    end
    render json: @posts.as_json(include: { tags: { only: [:id, :name, :category] } })
  end

  # GET /posts/1
  def show
    @post = Post.includes(:tags).find(params[:id])
    render json: @post.as_json(include: { tags: { only: [:id, :name, :category] } })
  end

  # POST /posts
  def create
    @post = Post.new(post_params)

    if @post.save
      render json: @post, status: :created, location: @post
    else
      render json: @post.errors, status: :unprocessable_entity
    end
  end

  # PATCH/PUT /posts/1
  def update
    if @post.update(post_params)
      render json: @post
    else
      render json: @post.errors, status: :unprocessable_entity
    end
  end

  # DELETE /posts/1
  def destroy
    @post.destroy!
  end

  private
    # Use callbacks to share common setup or constraints between actions.
    def set_post
      @post = Post.find(params.expect(:id))
    end

    # Only allow a list of trusted parameters through.
    def post_params
      params.expect(post: [ :title, :body ])
    end
end
